/*******************************************************************************
 * Copyright 2013 Thomas Letsch (contact@thomas-letsch.de)
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *   http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 ******************************************************************************/
package org.vaadin.addons.javaee.page;

import javax.enterprise.context.Dependent;

import org.vaadin.addons.javaee.buttons.ButtonBar;
import org.vaadin.addons.javaee.buttons.CancelButton;
import org.vaadin.addons.javaee.buttons.OkButton;
import org.vaadin.addons.javaee.buttons.handler.CanHandleCancelButton;
import org.vaadin.addons.javaee.buttons.handler.CanHandleOkButton;
import org.vaadin.addons.javaee.container.EntityContainer;
import org.vaadin.addons.javaee.container.EntityItem;
import org.vaadin.addons.javaee.form.BasicEditForm;
import org.vaadin.addons.javaee.i18n.TranslationKeys;
import org.vaadin.addons.javaee.i18n.TranslationService;
import org.vaadin.addons.javaee.table.BasicEntityTable;

import com.vaadin.ui.UI;
import com.vaadin.ui.VerticalLayout;
import com.vaadin.ui.Window;

@Dependent
public class AddEditDialog extends Window implements CanHandleOkButton, CanHandleCancelButton {

    private static final long serialVersionUID = 1L;

    private TranslationService translationService;

    protected BasicEditForm<?> form;

    private final EntityContainer<?> container;

    protected OkButton okButton;

    protected CancelButton cancelButton;

    private VerticalLayout content;

    public AddEditDialog(EntityContainer<?> container, BasicEditForm<?> form, TranslationService translationService) {
        this.translationService = translationService;
        this.container = container;
        this.form = form;

        setId(container.getEntityClass().getSimpleName() + "AddEditDialog");
        setCaption(translationService.getText(container.getEntityClass().getSimpleName()));
        setWidth(600, Unit.PIXELS);

        content = new VerticalLayout();
        setContent(content);
        content.addComponent(form);
        initButtons();
    }

    protected void initButtons() {
        ButtonBar buttons = new ButtonBar();
        OkButton okButton = new OkButton(this, translationService.getText(TranslationKeys.BUTTON_OK));
        buttons.addComponent(okButton);
        CancelButton cancelButton = new CancelButton(this, translationService.getText(TranslationKeys.BUTTON_CANCEL));
        buttons.addComponent(cancelButton);
        content.addComponent(buttons);
    }

    public void editSelected(BasicEntityTable<?> table) {
        Long itemId = table.getValue();
        edit(itemId);
    }

    @SuppressWarnings({ "rawtypes" })
    public void edit(Long itemId) {
        EntityItem entityItem = container.getItem(itemId);
        edit(entityItem);
    }

    @SuppressWarnings({ "rawtypes", "unchecked" })
    public void edit(EntityItem entityItem) {
        form.edit(entityItem);
        UI.getCurrent().addWindow(this);
        center();
    }

    public void editNew() {
        form.editNew();
        UI.getCurrent().addWindow(this);
        center();
    }

    @Override
    public void cancelClicked() {
        UI.getCurrent().removeWindow(this);
    }

    @Override
    public void okClicked() {
        form.save();
        UI.getCurrent().removeWindow(this);
    }

}
